@@ -1,3 +1,4 @@
+//go:build integration
 // +build integration
 
 package uatest
@@ -11,7 +12,13 @@ import (
 	"github.com/pascaldekloe/goe/verify"
 )
 
+type Complex struct {
+	i, j int64
+}
+
 func TestCallMethod(t *testing.T) {
+	ua.RegisterExtensionObject(ua.NewStringNodeID(2, "ComplexType"), new(Complex))
+
 	tests := []struct {
 		req *ua.CallMethodRequest
 		out []*ua.Variant
@@ -26,20 +33,42 @@ func TestCallMethod(t *testing.T) {
 			},
 			out: []*ua.Variant{ua.MustVariant(true)},
 		},
+		{
+			req: &ua.CallMethodRequest{
+				ObjectID: ua.NewStringNodeID(2, "main"),
+				MethodID: ua.NewStringNodeID(2, "square"),
+				InputArguments: []*ua.Variant{
+					ua.MustVariant(int64(3)),
+				},
+			},
+			out: []*ua.Variant{ua.MustVariant(int64(9))},
+		},
+		{
+			req: &ua.CallMethodRequest{
+				ObjectID: ua.NewStringNodeID(2, "main"),
+				MethodID: ua.NewStringNodeID(2, "sumOfSquare"),
+				InputArguments: []*ua.Variant{
+					ua.MustVariant(ua.NewExtensionObject(&Complex{3, 8})),
+				},
+			},
+			out: []*ua.Variant{ua.MustVariant(int64(9 + 64))},
+		},
 	}
 
+	ctx := context.Background()
+
 	srv := NewServer("method_server.py")
 	defer srv.Close()
 
 	c := opcua.NewClient(srv.Endpoint, srv.Opts...)
-	if err := c.Connect(context.Background()); err != nil {
+	if err := c.Connect(ctx); err != nil {
 		t.Fatal(err)
 	}
-	defer c.Close()
+	defer c.CloseWithContext(ctx)
 
 	for _, tt := range tests {
 		t.Run(tt.req.ObjectID.String(), func(t *testing.T) {
-			resp, err := c.Call(tt.req)
+			resp, err := c.CallWithContext(ctx, tt.req)
 			if err != nil {
 				t.Fatal(err)
 			}
