@@ -206,12 +206,8 @@ func TestVariant(t *testing.T) {
 			},
 		},
 		{
-			Name: "ExpandedNodeID",
-			Struct: MustVariant(NewExpandedNodeID(
-				true, false,
-				NewTwoByteNodeID(0xff),
-				"foobar", 0,
-			)),
+			Name:   "ExpandedNodeID",
+			Struct: MustVariant(NewExpandedNodeID(NewTwoByteNodeID(0xff), "foobar", 0)),
 			Bytes: []byte{
 				// variant encoding mask
 				0x12,
@@ -452,6 +448,35 @@ func TestVariant(t *testing.T) {
 				0x01, 0x00, 0x00, 0x00,
 			},
 		},
+		{
+			Name:   "ByteArray",
+			Struct: MustVariant(ByteArray{0x01, 0x02, 0x03}),
+			Bytes: []byte{
+				// variant encoding mask
+				0x83,
+				// array length
+				0x03, 0x00, 0x00, 0x00,
+				// array values
+				0x01, 0x02, 0x03,
+			},
+		},
+		{
+			Name:   "[]ByteArray",
+			Struct: MustVariant([]ByteArray{{0x01}, {0x02}, {0x03}}),
+			Bytes: []byte{
+				// variant encoding mask
+				0xc3,
+				// array length
+				0x03, 0x00, 0x00, 0x00,
+				// array values
+				0x01, 0x02, 0x03,
+				// array dimensions length
+				0x02, 0x00, 0x00, 0x00,
+				// array dimensions
+				0x03, 0x00, 0x00, 0x00,
+				0x01, 0x00, 0x00, 0x00,
+			},
+		},
 	}
 	RunCodecTest(t, cases)
 }
@@ -637,6 +662,24 @@ func TestSet(t *testing.T) {
 				value:       [][]byte{{0xca, 0xfe}, {0xaf, 0xfe}},
 			},
 		},
+		{
+			v: ByteArray{0xca, 0xfe},
+			va: &Variant{
+				mask:        byte(VariantArrayValues | TypeIDByte),
+				arrayLength: 2,
+				value:       ByteArray{0xca, 0xfe},
+			},
+		},
+		{
+			v: []ByteArray{{0xca}, {0xfe}},
+			va: &Variant{
+				mask:                  byte(VariantArrayDimensions | VariantArrayValues | TypeIDByte),
+				arrayLength:           2,
+				arrayDimensionsLength: 2,
+				arrayDimensions:       []int32{2, 1},
+				value:                 []ByteArray{{0xca}, {0xfe}},
+			},
+		},
 		{
 			v: int32(5),
 			va: &Variant{
@@ -688,7 +731,11 @@ func TestSet(t *testing.T) {
 			if got, want := err, tt.err; got != want {
 				t.Fatalf("got error %v want %v", got, want)
 			}
-			verify.Values(t, "variant", va, tt.va)
+			verify.Values(t, "variant.mask", va.mask, tt.va.mask)
+			verify.Values(t, "variant.arrayLength", va.arrayLength, tt.va.arrayLength)
+			verify.Values(t, "variant.arrayDimensionsLength", va.arrayDimensionsLength, tt.va.arrayDimensionsLength)
+			verify.Values(t, "variant.arrayDimensions", va.arrayDimensions, tt.va.arrayDimensions)
+			verify.Values(t, "variant.value", va.value, tt.va.value)
 		})
 	}
 }
@@ -745,6 +792,18 @@ func TestSliceDim(t *testing.T) {
 			dim: []int32{3, 0},
 			len: 0,
 		},
+		{
+			v:   ByteArray{},
+			et:  reflect.TypeOf(byte(0)),
+			dim: []int32{0},
+			len: 0,
+		},
+		{
+			v:   []ByteArray{{}, {}, {}},
+			et:  reflect.TypeOf(byte(0)),
+			dim: []int32{3, 0},
+			len: 0,
+		},
 
 		// error flows
 		{
@@ -910,6 +969,18 @@ func TestVariantValueHelpers(t *testing.T) {
 			fn:   func(v *Variant) interface{} { return v.Uint() },
 		},
 
+		// ByteArray
+		{
+			v:    false,
+			want: (ByteArray)(nil),
+			fn:   func(v *Variant) interface{} { return v.ByteArray() },
+		},
+		{
+			v:    ByteArray("abc"),
+			want: ByteArray("abc"),
+			fn:   func(v *Variant) interface{} { return v.ByteArray() },
+		},
+
 		// ByteString
 		{
 			v:    false,
@@ -1108,6 +1179,13 @@ func TestVariantValueHelpers(t *testing.T) {
 			fn:   func(v *Variant) interface{} { return v.ByteString() },
 		},
 
+		// []ByteArray
+		{
+			v:    []ByteArray{{'x', 'y', 'z'}},
+			want: (ByteArray)(nil),
+			fn:   func(v *Variant) interface{} { return v.ByteArray() },
+		},
+
 		// []*DataValue
 		{
 			v:    []*DataValue{{Status: StatusBad}},
