@@ -20,7 +20,7 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
-	"io/ioutil"
+	"os"
 	"reflect"
 	"testing"
 	"time"
@@ -29,10 +29,9 @@ import (
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/mock"
 	"github.com/stretchr/testify/require"
-	"github.com/uber/jaeger-lib/metrics"
-	"github.com/uber/jaeger-lib/metrics/metricstest"
 	"go.uber.org/zap"
 
+	"github.com/jaegertracing/jaeger/internal/metricstest"
 	"github.com/jaegertracing/jaeger/model"
 	"github.com/jaegertracing/jaeger/pkg/es/mocks"
 	"github.com/jaegertracing/jaeger/pkg/testutils"
@@ -40,6 +39,8 @@ import (
 	"github.com/jaegertracing/jaeger/storage/spanstore"
 )
 
+const defaultMaxDocCount = 10_000
+
 var exampleESSpan = []byte(
 	`{
 	   "traceID": "1",
@@ -101,6 +102,7 @@ func withSpanReader(fn func(r *spanReaderTest)) {
 			MaxSpanAge:        0,
 			IndexPrefix:       "",
 			TagDotReplacement: "@",
+			MaxDocCount:       defaultMaxDocCount,
 		}),
 	}
 	fn(r)
@@ -129,14 +131,34 @@ func withArchiveSpanReader(readAlias bool, fn func(r *spanReaderTest)) {
 var _ spanstore.Reader = &SpanReader{} // check API conformance
 
 func TestNewSpanReader(t *testing.T) {
-	client := &mocks.Client{}
-	reader := NewSpanReader(SpanReaderParams{
-		Client:         client,
-		Logger:         zap.NewNop(),
-		MaxSpanAge:     0,
-		MetricsFactory: metrics.NullFactory,
-		IndexPrefix:    ""})
-	assert.NotNil(t, reader)
+	tests := []struct {
+		name       string
+		params     SpanReaderParams
+		maxSpanAge time.Duration
+	}{
+		{
+			name: "no rollover",
+			params: SpanReaderParams{
+				MaxSpanAge: time.Hour * 72,
+			},
+			maxSpanAge: time.Hour * 72,
+		},
+		{
+			name: "rollover enabled",
+			params: SpanReaderParams{
+				MaxSpanAge:          time.Hour * 72,
+				UseReadWriteAliases: true,
+			},
+			maxSpanAge: time.Hour * 24 * 365 * 50,
+		},
+	}
+	for _, test := range tests {
+		t.Run(test.name, func(t *testing.T) {
+			reader := NewSpanReader(test.params)
+			require.NotNil(t, reader)
+			assert.Equal(t, test.maxSpanAge, reader.maxSpanAge)
+		})
+	}
 }
 
 func TestSpanReaderIndices(t *testing.T) {
@@ -144,37 +166,127 @@ func TestSpanReaderIndices(t *testing.T) {
 	logger, _ := testutils.NewLogger()
 	metricsFactory := metricstest.NewFactory(0)
 	date := time.Date(2019, 10, 10, 5, 0, 0, 0, time.UTC)
-	dateFormat := date.UTC().Format("2006-01-02")
+	spanDataLayout := "2006-01-02-15"
+	serviceDataLayout := "2006-01-02"
+	spanDataLayoutFormat := date.UTC().Format(spanDataLayout)
+	serviceDataLayoutFormat := date.UTC().Format(serviceDataLayout)
+
 	testCases := []struct {
-		index  string
-		params SpanReaderParams
+		indices []string
+		params  SpanReaderParams
 	}{
-		{params: SpanReaderParams{Client: client, Logger: logger, MetricsFactory: metricsFactory,
-			IndexPrefix: "", Archive: false},
-			index: spanIndex + dateFormat},
-		{params: SpanReaderParams{Client: client, Logger: logger, MetricsFactory: metricsFactory,
-			IndexPrefix: "", UseReadWriteAliases: true},
-			index: spanIndex + "read"},
-		{params: SpanReaderParams{Client: client, Logger: logger, MetricsFactory: metricsFactory,
-			IndexPrefix: "foo:", Archive: false},
-			index: "foo:" + indexPrefixSeparator + spanIndex + dateFormat},
-		{params: SpanReaderParams{Client: client, Logger: logger, MetricsFactory: metricsFactory,
-			IndexPrefix: "foo:", UseReadWriteAliases: true},
-			index: "foo:-" + spanIndex + "read"},
-		{params: SpanReaderParams{Client: client, Logger: logger, MetricsFactory: metricsFactory,
-			IndexPrefix: "", Archive: true},
-			index: spanIndex + archiveIndexSuffix},
-		{params: SpanReaderParams{Client: client, Logger: logger, MetricsFactory: metricsFactory,
-			IndexPrefix: "foo:", Archive: true},
-			index: "foo:" + indexPrefixSeparator + spanIndex + archiveIndexSuffix},
-		{params: SpanReaderParams{Client: client, Logger: logger, MetricsFactory: metricsFactory,
-			IndexPrefix: "foo:", Archive: true, UseReadWriteAliases: true},
-			index: "foo:" + indexPrefixSeparator + spanIndex + archiveReadIndexSuffix},
+		{
+			params: SpanReaderParams{
+				Client: client, Logger: logger, MetricsFactory: metricsFactory,
+				IndexPrefix: "", Archive: false, SpanIndexDateLayout: spanDataLayout, ServiceIndexDateLayout: serviceDataLayout,
+			},
+			indices: []string{spanIndex + spanDataLayoutFormat, serviceIndex + serviceDataLayoutFormat},
+		},
+		{
+			params: SpanReaderParams{
+				Client: client, Logger: logger, MetricsFactory: metricsFactory,
+				IndexPrefix: "", UseReadWriteAliases: true,
+			},
+			indices: []string{spanIndex + "read", serviceIndex + "read"},
+		},
+		{
+			params: SpanReaderParams{
+				Client: client, Logger: logger, MetricsFactory: metricsFactory,
+				IndexPrefix: "foo:", Archive: false, SpanIndexDateLayout: spanDataLayout, ServiceIndexDateLayout: serviceDataLayout,
+			},
+			indices: []string{"foo:" + indexPrefixSeparator + spanIndex + spanDataLayoutFormat, "foo:" + indexPrefixSeparator + serviceIndex + serviceDataLayoutFormat},
+		},
+		{
+			params: SpanReaderParams{
+				Client: client, Logger: logger, MetricsFactory: metricsFactory,
+				IndexPrefix: "foo:", UseReadWriteAliases: true,
+			},
+			indices: []string{"foo:-" + spanIndex + "read", "foo:-" + serviceIndex + "read"},
+		},
+		{
+			params: SpanReaderParams{
+				Client: client, Logger: logger, MetricsFactory: metricsFactory,
+				IndexPrefix: "", Archive: true,
+			},
+			indices: []string{spanIndex + archiveIndexSuffix, serviceIndex + archiveIndexSuffix},
+		},
+		{
+			params: SpanReaderParams{
+				Client: client, Logger: logger, MetricsFactory: metricsFactory,
+				IndexPrefix: "foo:", Archive: true,
+			},
+			indices: []string{"foo:" + indexPrefixSeparator + spanIndex + archiveIndexSuffix, "foo:" + indexPrefixSeparator + serviceIndex + archiveIndexSuffix},
+		},
+		{
+			params: SpanReaderParams{
+				Client: client, Logger: logger, MetricsFactory: metricsFactory,
+				IndexPrefix: "foo:", Archive: true, UseReadWriteAliases: true,
+			},
+			indices: []string{"foo:" + indexPrefixSeparator + spanIndex + archiveReadIndexSuffix, "foo:" + indexPrefixSeparator + serviceIndex + archiveReadIndexSuffix},
+		},
+		{
+			params: SpanReaderParams{
+				Client: client, Logger: logger, MetricsFactory: metricsFactory,
+				IndexPrefix: "", Archive: false, RemoteReadClusters: []string{"cluster_one", "cluster_two"}, SpanIndexDateLayout: spanDataLayout, ServiceIndexDateLayout: serviceDataLayout,
+			},
+			indices: []string{
+				spanIndex + spanDataLayoutFormat,
+				"cluster_one:" + spanIndex + spanDataLayoutFormat,
+				"cluster_two:" + spanIndex + spanDataLayoutFormat,
+				serviceIndex + serviceDataLayoutFormat,
+				"cluster_one:" + serviceIndex + serviceDataLayoutFormat,
+				"cluster_two:" + serviceIndex + serviceDataLayoutFormat,
+			},
+		},
+		{
+			params: SpanReaderParams{
+				Client: client, Logger: logger, MetricsFactory: metricsFactory,
+				IndexPrefix: "", Archive: true, RemoteReadClusters: []string{"cluster_one", "cluster_two"},
+			},
+			indices: []string{
+				spanIndex + archiveIndexSuffix,
+				"cluster_one:" + spanIndex + archiveIndexSuffix,
+				"cluster_two:" + spanIndex + archiveIndexSuffix,
+				serviceIndex + archiveIndexSuffix,
+				"cluster_one:" + serviceIndex + archiveIndexSuffix,
+				"cluster_two:" + serviceIndex + archiveIndexSuffix,
+			},
+		},
+		{
+			params: SpanReaderParams{
+				Client: client, Logger: logger, MetricsFactory: metricsFactory,
+				IndexPrefix: "", Archive: false, UseReadWriteAliases: true, RemoteReadClusters: []string{"cluster_one", "cluster_two"},
+			},
+			indices: []string{
+				spanIndex + "read",
+				"cluster_one:" + spanIndex + "read",
+				"cluster_two:" + spanIndex + "read",
+				serviceIndex + "read",
+				"cluster_one:" + serviceIndex + "read",
+				"cluster_two:" + serviceIndex + "read",
+			},
+		},
+		{
+			params: SpanReaderParams{
+				Client: client, Logger: logger, MetricsFactory: metricsFactory,
+				IndexPrefix: "", Archive: true, UseReadWriteAliases: true, RemoteReadClusters: []string{"cluster_one", "cluster_two"},
+			},
+			indices: []string{
+				spanIndex + archiveReadIndexSuffix,
+				"cluster_one:" + spanIndex + archiveReadIndexSuffix,
+				"cluster_two:" + spanIndex + archiveReadIndexSuffix,
+				serviceIndex + archiveReadIndexSuffix,
+				"cluster_one:" + serviceIndex + archiveReadIndexSuffix,
+				"cluster_two:" + serviceIndex + archiveReadIndexSuffix,
+			},
+		},
 	}
 	for _, testCase := range testCases {
 		r := NewSpanReader(testCase.params)
-		actual := r.timeRangeIndices(r.spanIndexPrefix, date, date)
-		assert.Equal(t, []string{testCase.index}, actual)
+
+		actualSpan := r.timeRangeIndices(r.spanIndexPrefix, r.spanIndexDateLayout, date, date, -1*time.Hour)
+		actualService := r.timeRangeIndices(r.serviceIndexPrefix, r.serviceIndexDateLayout, date, date, -24*time.Hour)
+		assert.Equal(t, testCase.indices, append(actualSpan, actualService...))
 	}
 }
 
@@ -216,11 +328,17 @@ func TestSpanReader_multiRead_followUp_query(t *testing.T) {
 		spanBytesID2, err := json.Marshal(spanID2)
 		require.NoError(t, err)
 
-		id1Query := elastic.NewTermQuery("traceID", model.TraceID{High: 0, Low: 1}.String())
+		traceID1Query := elastic.NewBoolQuery().Should(
+			elastic.NewTermQuery(traceIDField, model.TraceID{High: 0, Low: 1}.String()).Boost(2),
+			elastic.NewTermQuery(traceIDField, fmt.Sprintf("%x", 1)))
+		id1Query := elastic.NewBoolQuery().Must(traceID1Query)
 		id1Search := elastic.NewSearchRequest().
 			IgnoreUnavailable(true).
 			Source(r.reader.sourceFn(id1Query, model.TimeAsEpochMicroseconds(date.Add(-time.Hour))))
-		id2Query := elastic.NewTermQuery("traceID", model.TraceID{High: 0, Low: 2}.String())
+		traceID2Query := elastic.NewBoolQuery().Should(
+			elastic.NewTermQuery(traceIDField, model.TraceID{High: 0, Low: 2}.String()).Boost(2),
+			elastic.NewTermQuery(traceIDField, fmt.Sprintf("%x", 2)))
+		id2Query := elastic.NewBoolQuery().Must(traceID2Query)
 		id2Search := elastic.NewSearchRequest().
 			IgnoreUnavailable(true).
 			Source(r.reader.sourceFn(id2Query, model.TimeAsEpochMicroseconds(date.Add(-time.Hour))))
@@ -428,6 +546,7 @@ func TestSpanReaderFindIndices(t *testing.T) {
 	today := time.Date(1995, time.April, 21, 4, 12, 19, 95, time.UTC)
 	yesterday := today.AddDate(0, 0, -1)
 	twoDaysAgo := today.AddDate(0, 0, -2)
+	dateLayout := "2006-01-02"
 
 	testCases := []struct {
 		startTime time.Time
@@ -438,30 +557,30 @@ func TestSpanReaderFindIndices(t *testing.T) {
 			startTime: today.Add(-time.Millisecond),
 			endTime:   today,
 			expected: []string{
-				indexWithDate(spanIndex, today),
+				indexWithDate(spanIndex, dateLayout, today),
 			},
 		},
 		{
 			startTime: today.Add(-13 * time.Hour),
 			endTime:   today,
 			expected: []string{
-				indexWithDate(spanIndex, today),
-				indexWithDate(spanIndex, yesterday),
+				indexWithDate(spanIndex, dateLayout, today),
+				indexWithDate(spanIndex, dateLayout, yesterday),
 			},
 		},
 		{
 			startTime: today.Add(-48 * time.Hour),
 			endTime:   today,
 			expected: []string{
-				indexWithDate(spanIndex, today),
-				indexWithDate(spanIndex, yesterday),
-				indexWithDate(spanIndex, twoDaysAgo),
+				indexWithDate(spanIndex, dateLayout, today),
+				indexWithDate(spanIndex, dateLayout, yesterday),
+				indexWithDate(spanIndex, dateLayout, twoDaysAgo),
 			},
 		},
 	}
 	withSpanReader(func(r *spanReaderTest) {
 		for _, testCase := range testCases {
-			actual := r.reader.timeRangeIndices(spanIndex, testCase.startTime, testCase.endTime)
+			actual := r.reader.timeRangeIndices(spanIndex, dateLayout, testCase.startTime, testCase.endTime, -24*time.Hour)
 			assert.EqualValues(t, testCase.expected, actual)
 		}
 	})
@@ -469,7 +588,7 @@ func TestSpanReaderFindIndices(t *testing.T) {
 
 func TestSpanReader_indexWithDate(t *testing.T) {
 	withSpanReader(func(r *spanReaderTest) {
-		actual := indexWithDate(spanIndex, time.Date(1995, time.April, 21, 4, 21, 19, 95, time.UTC))
+		actual := indexWithDate(spanIndex, "2006-01-02", time.Date(1995, time.April, 21, 4, 21, 19, 95, time.UTC))
 		assert.Equal(t, "jaeger-span-1995-04-21", actual)
 	})
 }
@@ -487,54 +606,72 @@ func testGet(typ string, t *testing.T) {
 		caption        string
 		searchResult   *elastic.SearchResult
 		searchError    error
-		expectedError  string
-		expectedOutput []string
+		expectedError  func() string
+		expectedOutput map[string]interface{}
 	}{
 		{
-			caption:        typ + " full behavior",
-			searchResult:   &elastic.SearchResult{Aggregations: elastic.Aggregations(goodAggregations)},
-			expectedOutput: []string{"123"},
+			caption:      typ + " full behavior",
+			searchResult: &elastic.SearchResult{Aggregations: elastic.Aggregations(goodAggregations)},
+			expectedOutput: map[string]interface{}{
+				operationsAggregation: []spanstore.Operation{{Name: "123"}},
+				"default":             []string{"123"},
+			},
+			expectedError: func() string {
+				return ""
+			},
 		},
 		{
-			caption:       typ + " search error",
-			searchError:   errors.New("Search failure"),
-			expectedError: "Search service failed: Search failure",
+			caption:     typ + " search error",
+			searchError: errors.New("Search failure"),
+			expectedError: func() string {
+				if typ == operationsAggregation {
+					return "search operations failed: Search failure"
+				}
+				return "search services failed: Search failure"
+			},
 		},
 		{
-			caption:       typ + " search error",
-			searchResult:  &elastic.SearchResult{Aggregations: elastic.Aggregations(badAggregations)},
-			expectedError: "Could not find aggregation of " + typ,
+			caption:      typ + " search error",
+			searchResult: &elastic.SearchResult{Aggregations: elastic.Aggregations(badAggregations)},
+			expectedError: func() string {
+				return "could not find aggregation of " + typ
+			},
 		},
 	}
+
 	for _, tc := range testCases {
 		testCase := tc
 		t.Run(testCase.caption, func(t *testing.T) {
 			withSpanReader(func(r *spanReaderTest) {
 				mockSearchService(r).Return(testCase.searchResult, testCase.searchError)
-
 				actual, err := returnSearchFunc(typ, r)
-				if testCase.expectedError != "" {
-					require.EqualError(t, err, testCase.expectedError)
+				if testCase.expectedError() != "" {
+					require.EqualError(t, err, testCase.expectedError())
 					assert.Nil(t, actual)
+				} else if expectedOutput, ok := testCase.expectedOutput[typ]; ok {
+					assert.EqualValues(t, expectedOutput, actual)
 				} else {
-					require.NoError(t, err)
-					assert.EqualValues(t, testCase.expectedOutput, actual)
+					assert.EqualValues(t, testCase.expectedOutput["default"], actual)
 				}
 			})
 		})
 	}
 }
 
-func returnSearchFunc(typ string, r *spanReaderTest) ([]string, error) {
+func returnSearchFunc(typ string, r *spanReaderTest) (interface{}, error) {
 	if typ == servicesAggregation {
 		return r.reader.GetServices(context.Background())
 	} else if typ == operationsAggregation {
-		return r.reader.GetOperations(context.Background(), "someService")
+		return r.reader.GetOperations(
+			context.Background(),
+			spanstore.OperationQueryParameters{ServiceName: "someService"},
+		)
 	} else if typ == traceIDAggregation {
 		return r.reader.findTraceIDs(context.Background(), &spanstore.TraceQueryParameters{})
 	}
 	return nil, errors.New("Specify services, operations, traceIDs only")
 }
+
 func TestSpanReader_bucketToStringArray(t *testing.T) {
 	withSpanReader(func(r *spanReaderTest) {
 		buckets := make([]*elastic.AggregationBucketKeyItem, 3)
@@ -557,7 +694,7 @@ func TestSpanReader_bucketToStringArrayError(t *testing.T) {
 		buckets[2] = &elastic.AggregationBucketKeyItem{Key: 2}
 
 		_, err := bucketToStringArray(buckets)
-		assert.EqualError(t, err, "Non-string key found in aggregation")
+		assert.EqualError(t, err, "non-string key found in aggregation")
 	})
 }
 
@@ -783,17 +920,28 @@ func TestSpanReader_FindTracesSpanCollectionFailure(t *testing.T) {
 }
 
 func TestFindTraceIDs(t *testing.T) {
-	testGet(traceIDAggregation, t)
+	testCases := []struct {
+		aggregrationID string
+	}{
+		{traceIDAggregation},
+		{servicesAggregation},
+		{operationsAggregation},
+	}
+	for _, testCase := range testCases {
+		t.Run(testCase.aggregrationID, func(t *testing.T) {
+			testGet(testCase.aggregrationID, t)
+		})
+	}
 }
 
 func TestTraceIDsStringsToModelsConversion(t *testing.T) {
 	traceIDs, err := convertTraceIDsStringsToModels([]string{"1", "2", "3"})
 	assert.NoError(t, err)
 	assert.Equal(t, 3, len(traceIDs))
-	assert.Equal(t, "1", traceIDs[0].String())
+	assert.Equal(t, model.NewTraceID(0, 1), traceIDs[0])
 
 	traceIDs, err = convertTraceIDsStringsToModels([]string{"dsfjsdklfjdsofdfsdbfkgbgoaemlrksdfbsdofgerjl"})
-	assert.EqualError(t, err, "Making traceID from string 'dsfjsdklfjdsofdfsdbfkgbgoaemlrksdfbsdofgerjl' failed: TraceID cannot be longer than 32 hex characters: dsfjsdklfjdsofdfsdbfkgbgoaemlrksdfbsdofgerjl")
+	assert.EqualError(t, err, "making traceID from string 'dsfjsdklfjdsofdfsdbfkgbgoaemlrksdfbsdofgerjl' failed: TraceID cannot be longer than 32 hex characters: dsfjsdklfjdsofdfsdbfkgbgoaemlrksdfbsdofgerjl")
 	assert.Equal(t, 0, len(traceIDs))
 }
 
@@ -814,15 +962,23 @@ func mockArchiveMultiSearchService(r *spanReaderTest, indexName string) *mock.Ca
 	return multiSearchService.On("Do", mock.AnythingOfType("*context.valueCtx"))
 }
 
+// matchTermsAggregation uses reflection to match the size attribute of the TermsAggregation; neither
+// attributes nor getters are exported by TermsAggregation.
+func matchTermsAggregation(termsAgg *elastic.TermsAggregation) bool {
+	val := reflect.ValueOf(termsAgg).Elem()
+	sizeVal := val.FieldByName("size").Elem().Int()
+	return sizeVal == defaultMaxDocCount
+}
+
 func mockSearchService(r *spanReaderTest) *mock.Call {
 	searchService := &mocks.SearchService{}
 	searchService.On("Query", mock.Anything).Return(searchService)
 	searchService.On("IgnoreUnavailable", mock.AnythingOfType("bool")).Return(searchService)
-	searchService.On("Size", mock.MatchedBy(func(i int) bool {
-		return i == 0 || i == defaultDocCount
+	searchService.On("Size", mock.MatchedBy(func(size int) bool {
+		return size == 0 // Aggregations apply size (bucket) limits in their own query objects, and do not apply at the parent query level.
 	})).Return(searchService)
-	searchService.On("Aggregation", stringMatcher(servicesAggregation), mock.AnythingOfType("*elastic.TermsAggregation")).Return(searchService)
-	searchService.On("Aggregation", stringMatcher(operationsAggregation), mock.AnythingOfType("*elastic.TermsAggregation")).Return(searchService)
+	searchService.On("Aggregation", stringMatcher(servicesAggregation), mock.MatchedBy(matchTermsAggregation)).Return(searchService)
+	searchService.On("Aggregation", stringMatcher(operationsAggregation), mock.MatchedBy(matchTermsAggregation)).Return(searchService)
 	searchService.On("Aggregation", stringMatcher(traceIDAggregation), mock.AnythingOfType("*elastic.TermsAggregation")).Return(searchService)
 	r.client.On("Search", mock.AnythingOfType("string"), mock.AnythingOfType("string"), mock.AnythingOfType("string"), mock.AnythingOfType("string"), mock.AnythingOfType("string")).Return(searchService)
 	return searchService.On("Do", mock.MatchedBy(func(ctx context.Context) bool {
@@ -847,7 +1003,7 @@ func TestTraceQueryParameterValidation(t *testing.T) {
 
 	tqp.ServiceName = serviceName
 
-	tqp.StartTimeMin = time.Time{} //time.Unix(0,0) doesn't work because timezones
+	tqp.StartTimeMin = time.Time{} // time.Unix(0,0) doesn't work because timezones
 	tqp.StartTimeMax = time.Time{}
 	err = validateQuery(tqp)
 	assert.EqualError(t, err, ErrStartAndEndTimeNotSet.Error())
@@ -924,8 +1080,7 @@ func TestSpanReader_buildFindTraceIDsQuery(t *testing.T) {
 }
 
 func TestSpanReader_buildDurationQuery(t *testing.T) {
-	expectedStr :=
-		`{ "range":
+	expectedStr := `{ "range":
 			{ "duration": { "include_lower": true,
 				        "include_upper": true,
 				        "from": 1000000,
@@ -950,9 +1105,8 @@ func TestSpanReader_buildDurationQuery(t *testing.T) {
 }
 
 func TestSpanReader_buildStartTimeQuery(t *testing.T) {
-	expectedStr :=
-		`{ "range":
-			{ "startTime": { "include_lower": true,
+	expectedStr := `{ "range":
+			{ "startTimeMillis": { "include_lower": true,
 				         "include_upper": true,
 				         "from": 1000000,
 				         "to": 2000000 }
@@ -968,8 +1122,8 @@ func TestSpanReader_buildStartTimeQuery(t *testing.T) {
 		expected := make(map[string]interface{})
 		json.Unmarshal([]byte(expectedStr), &expected)
 		// We need to do this because we cannot process a json into uint64.
-		expected["range"].(map[string]interface{})["startTime"].(map[string]interface{})["from"] = model.TimeAsEpochMicroseconds(startTimeMin)
-		expected["range"].(map[string]interface{})["startTime"].(map[string]interface{})["to"] = model.TimeAsEpochMicroseconds(startTimeMax)
+		expected["range"].(map[string]interface{})["startTimeMillis"].(map[string]interface{})["from"] = model.TimeAsEpochMicroseconds(startTimeMin) / 1000
+		expected["range"].(map[string]interface{})["startTimeMillis"].(map[string]interface{})["to"] = model.TimeAsEpochMicroseconds(startTimeMax) / 1000
 
 		assert.EqualValues(t, expected, actual)
 	})
@@ -1004,7 +1158,7 @@ func TestSpanReader_buildOperationNameQuery(t *testing.T) {
 }
 
 func TestSpanReader_buildTagQuery(t *testing.T) {
-	inStr, err := ioutil.ReadFile("fixtures/query_01.json")
+	inStr, err := os.ReadFile("fixtures/query_01.json")
 	require.NoError(t, err)
 	withSpanReader(func(r *spanReaderTest) {
 		tagQuery := r.reader.buildTagQuery("bat.foo", "spook")
@@ -1018,6 +1172,36 @@ func TestSpanReader_buildTagQuery(t *testing.T) {
 	})
 }
 
+func TestSpanReader_buildTagRegexQuery(t *testing.T) {
+	inStr, err := os.ReadFile("fixtures/query_02.json")
+	require.NoError(t, err)
+	withSpanReader(func(r *spanReaderTest) {
+		tagQuery := r.reader.buildTagQuery("bat.foo", "spo.*")
+		actual, err := tagQuery.Source()
+		require.NoError(t, err)
+
+		expected := make(map[string]interface{})
+		json.Unmarshal(inStr, &expected)
+
+		assert.EqualValues(t, expected, actual)
+	})
+}
+
+func TestSpanReader_buildTagRegexEscapedQuery(t *testing.T) {
+	inStr, err := os.ReadFile("fixtures/query_03.json")
+	require.NoError(t, err)
+	withSpanReader(func(r *spanReaderTest) {
+		tagQuery := r.reader.buildTagQuery("bat.foo", "spo\\*")
+		actual, err := tagQuery.Source()
+		require.NoError(t, err)
+
+		expected := make(map[string]interface{})
+		json.Unmarshal(inStr, &expected)
+
+		assert.EqualValues(t, expected, actual)
+	})
+}
+
 func TestSpanReader_GetEmptyIndex(t *testing.T) {
 	withSpanReader(func(r *spanReaderTest) {
 		mockSearchService(r).
@@ -1073,7 +1257,44 @@ func TestSpanReader_ArchiveTraces_ReadAlias(t *testing.T) {
 	})
 }
 
-func TestNewSpanReader2(t *testing.T) {
-	spanDate := time.Now().UTC().Format("2006")
-	fmt.Println(spanDate)
+func TestConvertTraceIDsStringsToModels(t *testing.T) {
+	ids, err := convertTraceIDsStringsToModels([]string{"1", "2", "01", "02", "001", "002"})
+	require.NoError(t, err)
+	assert.Equal(t, []model.TraceID{model.NewTraceID(0, 1), model.NewTraceID(0, 2)}, ids)
+	_, err = convertTraceIDsStringsToModels([]string{"1", "2", "01", "02", "001", "002", "blah"})
+	assert.Error(t, err)
+}
+
+func TestBuildTraceByIDQuery(t *testing.T) {
+	uintMax := ^uint64(0)
+	traceIDNoHigh := model.NewTraceID(0, 1)
+	traceIDHigh := model.NewTraceID(1, 1)
+	traceID := model.NewTraceID(uintMax, uintMax)
+	tests := []struct {
+		traceID model.TraceID
+		query   elastic.Query
+	}{
+		{
+			traceID: traceIDNoHigh,
+			query: elastic.NewBoolQuery().Should(
+				elastic.NewTermQuery(traceIDField, "0000000000000001").Boost(2),
+				elastic.NewTermQuery(traceIDField, "1"),
+			),
+		},
+		{
+			traceID: traceIDHigh,
+			query: elastic.NewBoolQuery().Should(
+				elastic.NewTermQuery(traceIDField, "00000000000000010000000000000001").Boost(2),
+				elastic.NewTermQuery(traceIDField, "10000000000000001"),
+			),
+		},
+		{
+			traceID: traceID,
+			query:   elastic.NewTermQuery(traceIDField, "ffffffffffffffffffffffffffffffff"),
+		},
+	}
+	for _, test := range tests {
+		q := buildTraceByIDQuery(test.traceID)
+		assert.Equal(t, test.query, q)
+	}
 }
